; input: file 2.txt, unsigned integers separated by \n
; output: screen, sorted integers
	.model small
	.STACK 100h

	.DATA
file	DB	"2.TXT",0
err1	DB	"Cannot Open File$"
err2	DB	"The file contains too many numbers$"
err3	DB	"The number is too long$"
err4	DB	"Fail reading file$"
num	DB	1000 DUP(0)	; array: maximum 100 numbers, each 10 long

	.CODE
	.STARTUP
	mov	ah, 3dh
	mov	al, 0
	lea	dx, file
	int	21h		; open file
	jc	notopen
	mov	bx, ax		; fileno is stored in bx
	mov	si, OFFSET num	; si: pointer to number array
	mov	di, 0		; di: offset in current number
	mov	cx, 1		; file read num
readnext:
	cmp	si, OFFSET num + SIZEOF num
	jge	toomany
	cmp	di, 10
	jge	toolong		; number max length: 10
	mov	ah, 3fh
	mov	dx, si
	add	dx, di		; read into [si+di]
	int	21h
	jc	readfail
	cmp	ax, 1
	jne	eof

	push	bx
	mov	bx, dx
	mov	al, BYTE PTR [bx]	; cannot address by dx
	pop	bx
	cmp	al, "0"
	jl	notnum
	cmp	al, "9"
	jg	notnum		; if not number, then next line
	add	di, 1
	jmp	readnext
notnum:
	push	bx
	mov	bx, dx
	mov	BYTE PTR [bx], 0	; clear the illegal char
	pop	bx
	cmp	di, 0		; di = number of chars in string
	je	readnext	; do not allow empty string
	push	bx
	mov	bx, si		; temp: bx = string base
	push	si
	add	si, 10
fixpos:
	sub	di, 1
	sub	si, 1		; si = where the last char should be put
	mov	al, BYTE PTR [bx+di]
	mov	BYTE PTR [bx+di], 0
	mov	BYTE PTR [si], al
	cmp	di, 0
	je	fixedpos
	jmp	fixpos
fixedpos:
	pop	si
	pop	bx
	add	si, 10
	jmp	readnext
eof:				; now si contains the index of last string
	mov	ah, 3eh
	int	21h		; close file
	mov	ax, OFFSET num
sort:
	cmp	ax, si
	jge	output
	mov	bx, ax
inner:
	cmp	bx, si
	jge	breakinner
	call	strcmp
	cmp	cx, 0
	jg	swap
swapped:
	add	bx, 10
	jmp	inner
breakinner:
	add	ax, 10
	jmp	sort

; swap two strings in ax and bx
; cx, dx, di will be overwritten
swap:
	mov	dx, si		; save si
	mov	si, ax
	mov	di, 0
swapnext:
	mov	cl, BYTE PTR [si]
	mov	ch, BYTE PTR [bx+di]
	mov	BYTE PTR [si], ch
	mov	BYTE PTR [bx+di], cl
	add	di, 1
	add	si, 1
	cmp	di, 10
	je	swapdone
	jmp	swapnext
swapdone:
	mov	si, dx
	jmp	swapped

; two strings are in ax and bx
; if ax < bx then cx = -1
; if ax > bx then cx = 1
; if ax = bx then cx = 0
; cx, dx, di will be overwritten
strcmp:
	mov	dx, si		; save si
	mov	si, ax
	mov	di, 0
strcmpnext:
	mov	cl, BYTE PTR [si]
	mov	ch, BYTE PTR [bx+di]
	cmp	cl, ch
	jg	strcmpgreat
	jl	strcmpless
	add	di, 1
	add	si, 1
	cmp	di, 10
	je	strcmpequal
	jmp	strcmpnext
strcmpequal:
	mov	cx, 0
	jmp	strcmpdone
strcmpgreat:
	mov	cx, 1
	jmp	strcmpdone
strcmpless:
	mov	cx, -1
strcmpdone:
	mov	si, dx
	ret

output:
	mov	bx, OFFSET num		; bx: outer pointer
	mov	di, 0			; di: inner offset
	mov	ah, 2			; always output
outputinner:
	mov	dl, BYTE PTR [bx+di]
	cmp	dl, "0"
	jl	printspace
	cmp	dl, "9"
	jg	printspace
	jmp	printdigit
printspace:
	mov	dl, " "
printdigit:
	int	21h
	add	di, 1
	cmp	di, 10
	je	outputouter
	jmp	outputinner
outputouter:
	add	bx, 10
	cmp	bx, si
	jge	return
	mov	dl, 0dh
	int	21h
	mov	dl, 0ah			; print New Line
	int	21h
	mov	di, 0
	jmp	outputinner

notopen:
	mov	ah, 09h
	lea	dx, err1
	int	21h
	jmp	return
toomany:
	mov	ah, 09h
	lea	dx, err2
	int	21h
	jmp	return
toolong:
	mov	ah, 09h
	lea	dx, err3
	int	21h
	jmp	return
readfail:
	mov	ah, 09h
	lea	dx, err4
	int	21h
	jmp	return
return:
	mov	ax, 4c00h	; return to dos
	int	21h
end
